// ==========================================================================
// Demo: Secure File Transfer
// Caution: no error check for the sake of briefness and understandability
// Tips:
// The ROCKEY5 dongle supports Secure File Transfer (SFT). The Remote Update Management (RUM) 
// function will verify the user's identity, but it cannot update the dongle algorithm because the 
// end user does not have access to the super password which is required to write to the dongle. 
// The SFT process allows the software developer to securely update dongle algorithms or data files 
// without compromising the super password.
// See the SFT operational overview described below:
// 1. The software developer generates the "cryptography file" by using with own dongle.
// 2. Transfer the file to the user;
// 3. The user transfers the cryptography file to dongle with the program provided by the software 
//    developer. The dongle converts the file to clear text file internally.
// We adopts the following security policies:
// 1. The algorithms of cryptography files and clear text files are completely confidential, not 
//    open to anyone;
// 2. The generation of cryptography files and clear text files is 100% performed inside dongle, 
//    no other external work is involved;
// 3. The encryption / decryption key is related to dongle super password and the management code. 
//    It is not necessary to worry that some one else could create a cryptography file valid in the 
//    end user's dongle
// 4. The content of the cryptography file can not be modified, the file ID, its attribute, and 
//    others are part of the cryptography file, and they are all protected by the file check 
//    mechanism.
// Attention::
// 1. The software developer can only generate the cryptography file after super password 
//    verification. The verified super password is part of the encryption key. That is to say, 
//    the super password of the end user should be the same with the super password which the 
//    software developer uses to generate the cryptography file.
// 2. During the cryptography file generation process, a temporary file with the same name will be 
//    created in the software developer's dongle, because the IC card can transfer at most data of 
//    250 bytes once, and many files are larger than that size. Therefore, dongle should have enough 
//    space for the temporary file generation. The file will not be deleted automatically after the 
//    "cryptography file" generation call and the return of the content; you need to delete it 
//    manually.
// ==========================================================================
// Here two security files will be created: an executable file and a data file. The function 
// of the executable file is to read and return the content of the data file. 

//
// Executable file readdata.c, input parameters are the internal file IDs of dongle to be accessed
// Output parameters are the contents of the internal file of dongle.
//
//
//void main(void)
//{
// word fileID;
// word fileSize;
// byte b_fileSize;
// byte xdata out[30];
// byte xdata result[100],ret;

// get_input(&fileID,0,1,1);
// open_file(&fileID, 0);
// get_file_infor(out, 1);
// fileSize = (out[5]<<8)|out[4];
// read_file(0,result,fileSize);
// b_fileSize=(unsigned char)fileSize;

//set_response(b_fileSize, result);
// exit();
//}
// ==========================================================================
unit main;

interface

uses
    Windows, Messages, SysUtils,  Classes, Graphics, Controls, Forms,
    Dialogs, StdCtrls ,Dic32u;

type
    TForm1 = class(TForm)
    Button1: TButton;
    List: TListBox;
    procedure Button1Click(Sender: TObject);
    private
        { Private declarations }
    public
        { Public declarations }
end;

var
    Form1: TForm1;

implementation



{$R *.dfm}

procedure TForm1.Button1Click(Sender: TObject);
var

str:string;
str2:string;
errcode:Integer;    // error code
num:Integer;	    // number of reader
hic:Integer;        // device handle
i:Integer;
cmddata:array[0..512] of Char;
buffer:array[0..256] of Char;
exesize:integer;                //encrypted exe-file size
datasize:integer;               //encrypted date-faile size
readsize:integer;
c_exefile:array[0..512] of Char;    //store encrypted exe-file
c_datafile:array[0..512] of Char;   //store encrypted date-faile

const
READDATA_SIZE = 276;
g_progReadData : array[0..READDATA_SIZE-1] of BYTE=
(
$02,$01,$04,$75,$0C,$00,$75,$0D,$00,$75,$0E,$08,$E4,$F5,$0F,$75,
$10,$01,$75,$11,$01,$12,$00,$A0,$75,$0C,$00,$75,$0D,$00,$75,$0E,
$08,$E4,$F5,$0F,$12,$00,$BF,$75,$0C,$01,$75,$0D,$00,$75,$0E,$00,
$75,$0F,$01,$12,$00,$ED,$90,$00,$04,$E0,$FE,$A3,$E0,$FD,$EE,$F5,
$0A,$ED,$F5,$0B,$E4,$F5,$0C,$F5,$0D,$75,$0E,$01,$75,$0F,$00,$75,
$10,$0C,$85,$0A,$11,$85,$0B,$12,$12,$00,$7D,$D3,$E5,$0B,$94,$FF,
$E5,$0A,$94,$00,$40,$04,$7F,$FF,$80,$02,$AF,$0B,$8F,$0C,$75,$0D,
$01,$75,$0E,$00,$75,$0F,$0C,$12,$00,$D6,$02,$01,$10,$12,$FF,$00,
$E5,$0C,$24,$00,$E5,$0D,$24,$01,$E5,$0E,$24,$02,$E5,$0F,$24,$03,
$E5,$10,$24,$04,$E5,$11,$24,$05,$E5,$12,$24,$06,$12,$FF,$03,$22,
$12,$FF,$00,$E5,$0C,$24,$00,$E5,$0D,$24,$01,$E5,$0E,$24,$02,$E5,
$0F,$24,$03,$E5,$10,$24,$04,$E5,$11,$24,$05,$12,$FF,$0B,$22,$12,
$FF,$00,$E5,$0C,$24,$00,$E5,$0D,$24,$01,$E5,$0E,$24,$02,$E5,$0F,
$24,$03,$12,$FF,$02,$22,$12,$FF,$00,$E5,$0D,$24,$00,$E5,$0E,$24,
$01,$E5,$0F,$24,$02,$E5,$0C,$24,$03,$12,$FF,$0A,$22,$12,$FF,$00,
$E5,$0C,$24,$00,$E5,$0D,$24,$01,$E5,$0E,$24,$02,$E5,$0F,$24,$03,
$12,$FF,$06,$22,$78,$7F,$E4,$F6,$D8,$FD,$75,$81,$12,$02,$00,$03,
$12,$FF,$0D,$22
);

begin
    List.Items.Clear();
 

    num := DIC_Find();
    for i:=0 to num-1 do
    begin
	    hic := DIC_Open(i, nil);
		if hic >= 0 then break;		// Find the first ROCKEY
	end;
    if i = num then exit;

    	// a. Change to super user status (Only super users are allowed to create executable files)
	for i:=0 to 7 do
        cmddata[i] := char($ff); // Super password is 8 FFs for demo ROCKEY
	errcode := DIC_Command(hic, CHECK_SUPER_PASS, cmddata);

    	// b. Format ROCKEY, all files and directorys will be cleared.
	//    You can set ATR and VOLUME here.
	DIC_Set(cmddata, VOLUME, BY_ARRAY, 0, pchar('DICSYSTEM'));
	DIC_Set(cmddata, ATR, BY_ARRAY, 0, pchar('DIC Co.'));
	errcode := DIC_Command(hic, FORMAT_CARD, cmddata);

	// c. Create the cryptography file of the executable file
	//    Difference between creating cryptography file and common file is that after setting the
	//    relevant file attributes FILE_DATA call is needed to transfer the file content.
	DIC_Set(cmddata, FILL, 512, 0, nil);	                			// Clear buffer
	DIC_Set(cmddata, FILE_ID, BY_VALUE, $0023, nil);				// File ID
	DIC_Set(cmddata, FILE_CLASS, BY_VALUE, $ff, nil);				// File class
	DIC_Set(cmddata, FILE_ATTRIBUTE, BY_VALUE, FILEATTR_EXEC, nil);			// File attr
	DIC_Set(cmddata, FILE_SIZE, BY_VALUE, READDATA_SIZE, nil);			// File size
	DIC_Set(cmddata, FILE_NAME, BY_ARRAY, 0, pchar('6088'));			// File name
	DIC_Set(cmddata, FILE_DATA, BY_ARRAY or READDATA_SIZE, 0, @g_progReadData);	// File content
    	errcode := DIC_Command(hic, CRYPTOTEXT_FILE, cmddata);                 		//encrypt exe-file
	exesize := DIC_Get(cmddata, FILE_DATA, BY_ARRAY, c_exefile);    
	
     	// d.  Create the cryptography file of the data file, data content is 0-9 repeated 8 times.
	for i:=0 to 79 do
        buffer[i] := Char(i mod 10);

    //{{file content
    str2:='initialize data file:';
    for i:=0 to 79 do
	begin
		FmtStr(str,'%x ', [byte(buffer[i])]);
        if (i mod 10) = 0 then
        begin
            List.Items.Add(str2);
            str2:=str;
        end
        else
        begin
            str2:=str2+str;
        end;
    end;
    List.Items.Add(str2);

	DIC_Set(cmddata, FILL, 256, 0, nil);					// Clear buffer
	DIC_Set(cmddata, FILE_ID, BY_VALUE, $0057, nil);			// File ID
	DIC_Set(cmddata, FILE_CLASS, BY_VALUE, $ff, nil);			// File class
	DIC_Set(cmddata, FILE_ATTRIBUTE, BY_VALUE, FILEATTR_NORMAL, nil);	// File attr
	DIC_Set(cmddata, FILE_SIZE, BY_VALUE, 80, nil);				// File size
	DIC_Set(cmddata, FILE_NAME, BY_ARRAY, 0, pchar('File1'));		// File name
	DIC_Set(cmddata, FILE_DATA, BY_ARRAY or 80, 0, buffer);			// File content
	errcode := DIC_Command(hic, CRYPTOTEXT_FILE, cmddata);          	//encrypt data file.
	datasize := DIC_Get(cmddata, FILE_DATA, BY_ARRAY, c_datafile);   

     	// e. Select the temporary file of the executable cryptography file($0023)
	DIC_Set(cmddata, FILE_ID, BY_VALUE, $0023, nil);
	errcode := DIC_Command(hic, SET_CURRENT_FILE, cmddata);

	// f. Delete temporary file of the executable cryptography file($0023)
	errcode := DIC_Command(hic, REMOVE_FILE, cmddata);

	// g. Select the temporary file of the data cryptography file($0057)
	DIC_Set(cmddata, FILE_ID, BY_VALUE, $0057, nil);
	errcode := DIC_Command(hic, SET_CURRENT_FILE, cmddata);

	// h. Delete the temporary file of the data cryptography file(0x0057)
	errcode := DIC_Command(hic, REMOVE_FILE, cmddata);

	// i. Lower the super password verified status to common user status, as it is unnecessary
	//    for super user to create the clear text file.
	errcode := DIC_Close(hic);

    //{{Run the following at client side, send c_exefile ,exesize,c_datafile, datasize to clients.
	errcode := DIC_Open(hic, nil);

	// j. Create the clear text file of the exe-file
	StrMove(cmddata, c_exefile, exesize);
	errcode := DIC_Command(hic, PLAINTEXT_FILE, cmddata);

	// k. Create the clear text file of the data file
	StrMove(cmddata, c_datafile, datasize);
	errcode := DIC_Command(hic, PLAINTEXT_FILE, cmddata);
    //}}Finish task at client side, step j is to test exe-file and data file

	// l. Run the executable file($0023), to get the content of the data file($0057)
	//    and return
	DIC_Set(buffer, 0, 2, $0057, nil);				// Input the file ID
	DIC_Set(cmddata, RUN_DATA, 2, $6088, buffer);			// Set the run data
	errcode := DIC_Command(hic, RUN, cmddata);
	readsize := DIC_Get(cmddata, RUN_DATA, BY_ARRAY, buffer);	// Fetch data after execution

    str2:='data file readed from ROCKEY:';
	for i:=0 to readsize-1 do
	begin
		FmtStr(str,'%x ', [byte(buffer[i])]);
        if (i mod 10) = 0 then
        begin
            List.Items.Add(str2);
            str2:=str;
        end
        else
            str2:=str2+str;
	end;
    List.Items.Add(str2);

    //m.Close
    errcode := DIC_Close(hic);
 
end;

end.


