! ==========================================================================
! Demo: Remote Update 
! Caution: no error check for the sake of briefness and understandability
! Tips:
! (1)  Remote Update Management (RUM) is a utility that may be utilized by ROCKEY5 developers 
!      to implement remote updates to their software protection and licensing systems. RUM is 
!      based on "one time use" passwords
! (2)  The RUM process is described below:( some steps is for developer, others for end user)
!     
!      1) "a.Set new basic remote update password"
!         The software developer must first initialize the ROCKEY5 dongle to set the Basic 
!         Remote Update Password (BRUP). This process requires use of the super password. Once 
!         the BRUP is set it can only be changed by reformatting the dongle by means of the 
!         super password. The initialized dongle may then be distributed to the end user.  
!      2) "b.Get remote update information and the current dongle hardware ID"
!         When software developers need to remote update their software, ROCKEY5 software at 
!         the user end gets the relevant information (the current remote update information and 
!         hardware ID) and transfers such data by some means to the software developer.
!      3) "d.Change the privilege to super user and acquire the next remote update password."
!         Upon receiving the pertinent information, the software developer gets the next update 
!         password for the end user and transfers the new remote update password to the end user.
!      4) "f.return to password not verified status"
!         next step is demo for end user, so return to password not verified status.
!      5) "g.Verify remote update password and check if the remote update password passes the 
!          verification."
!         After receiving the new update password from the software developer, the user can send 
!         the new password to dongle for verification. If it succeeds, the remote update tag of 
!         dongle is set, the program may check the tag status to determine the corresponding 
!         actions. Because the dongle internal program can also detect the tag, it is recommended 
!         that some part of the update process be performed by the internal program.
! (3) The remote update tag, RemoteTag, is not the update password, it is the information set by 
!     the software developer for identifying the current software update status. Usually, we 
!     recommend you write the software version here, thus the software developer can tell from 
!     which version the user intends to update. It is a 32-bit number, but its highest bit is used 
!     to show whether the "hardware ID" is used or not. If the bit is 1, the update password is 
!     related to the "Hardware ID" of dongle; if it is 0, it means the update password has nothing 
!     to do with the "hardware ID", it only makes reference to the previous update password. 
!     "one time password" simplifies update maintenance of the software developer.
! (4) Additionally, the RemoteTag and RemotePass provided to users are related to each other. The 
!     user can not modify either of the two. If the highest bit of RemoteTag is 1, the update 
!     password is only valid for some specific dongle. In the information the user offers to 
!     software developer, only the hardware ID and the previous update password are necessary. 
!     The last RemoteTag is merely a reference to the software developer, and it does not affect 
!     the next update information. The software developer should set a new RemoteTag as the update 
!     mark different from the previous ones for each update operation.
! ==========================================================================


!DEC$FREEFORM

INCLUDE 'head.F90'

integer*4 errcode 			! error code
integer*4 num 				! number of founded card reader(s)
integer*4 hic 				! handle of Rockey5 device.
integer*4 i 
integer*4 ret_code 
CHARACTER*256 reader_name
CHARACTER cmddata(256) 	
CHARACTER buffer(256) 	
CHARACTER*20  p_null
CHARACTER remotepass(8) 
integer*4 RemoteTag
integer*4 HardSerial

	num = DIC_Find() 
	do i=0 ,num-1
		hic = DIC_Open(i, loc(reader_name)) 
		if (hic >= 0) then
		exit
		endif		! Stop searching once one is found.
	enddo

	if (i == num)  then
        	stop
	endif

	! a.Set the new basic remote update password.
	ret_code=DIC_Set(loc(cmddata), REMOTE_TAG, BY_VALUE, 1, loc(p_null)) 	! Set remote tag to 1
	buffer='AAAAAAAA'c  ! set the new remote update password 8 As.
	ret_code= DIC_Set(loc(cmddata), REMOTE_PASS, BY_ARRAY, 0, loc(buffer))
	errcode = DIC_Command(hic, SET_REMOTE_INFO, loc(cmddata)) 		! Update remote update info

	! b.Read remote update info
	errcode = DIC_Command(hic, GET_REMOTE_INFO, loc(cmddata)) 
	RemoteTag = DIC_Get(loc(cmddata), REMOTE_TAG, BY_VALUE, loc(p_null)) 
	ret_code=DIC_Get(loc(cmddata), REMOTE_PASS, BY_ARRAY, loc(buffer)) 
	write(*,*) 'RemoteTag= ', RemoteTag 
	write(*,*) 'RemotePass=' 
	write(*,'(z3,z3,z3,z3,z3,z3,z3,z3)') buffer(1),buffer(2),buffer(3),buffer(4),buffer(5),buffer(6),buffer(7),buffer(8)
	DO i=1,8
	   remotepass(i)=buffer(i)
	enddo
	
	! c.Get hardware ID of the current dongle 
	errcode = DIC_Command(hic, GET_HARDWARE_INFO, loc(cmddata)) 	! Get dongle hardware ID
	HardSerial = DIC_Get(loc(cmddata), HARD_SERIAL, BY_VALUE, loc(p_null)) 

	! d.Change to super user
	do i=1 ,8
	cmddata(i)=char(255)  ! Super password is 8 FFs.
	enddo		
	errcode = DIC_Command(hic, CHECK_SUPER_PASS, loc(cmddata)) 

	! e.Get the next update password of the current one
	ret_code=DIC_Set(loc(cmddata), UPGRADE_REMOTE_TAG, BY_VALUE, 2, loc(p_null))	! The new update tag is 2
	ret_code=DIC_Set(loc(cmddata), UPGRADE_HARD_SERIAL, BY_VALUE, HardSerial, loc(p_null)) 
	do i=1,8
	    buffer(i)=remotepass(i)
	end do
	ret_code=DIC_Set(loc(cmddata), UPGRADE_REMOTE_PASS, BY_ARRAY, 0, loc(buffer)) 
	errcode = DIC_Command(hic, GET_UPGRADE_REMOTE_PASS, loc(cmddata)) 
	DO i=1,8
	   remotepass(i)=cmddata(i)    	! Transfer the new remote update password to remotepass
	enddo

	! f.Change the super user status to common user status
	errcode = DIC_Close(hic) 
	errcode = DIC_Open(hic, loc(p_null)) 

	! g.Verify the remote update password
	ret_code=DIC_Set(loc(cmddata), REMOTE_TAG, BY_VALUE, 2, loc(p_null)) 
	do i=1,8
	    buffer(i)=remotepass(i)
	end do	
	ret_code=DIC_Set(loc(cmddata), REMOTE_PASS, BY_ARRAY, 0, loc(buffer)) 
	errcode = DIC_Command(hic, CHECK_REMOTE_INFO, loc(cmddata)) 	! Verify remote update info

	! h.Check if remote update password verification succeeds
	errcode = DIC_Command(hic, GET_CARD_PRIVILEGE, loc(cmddata)) 	! Get privilege info
	ret_code=DIC_Get(loc(cmddata), REMOTEPASS_STATUS, BY_VALUE, loc(p_null))
	if (ret_code/= 0) then
		write(*,*)'Remote update password verification succeeded' 
	else 
	   write(*,*)'Remote update password verification failed' 
	end if

	! i.Read the new remote update information
	errcode = DIC_Command(hic, GET_REMOTE_INFO, loc(cmddata)) 
	RemoteTag = DIC_Get(loc(cmddata), REMOTE_TAG, BY_VALUE, loc(p_null)) 
	ret_code=DIC_Get(loc(cmddata), REMOTE_PASS, BY_ARRAY, loc(buffer)) 
	write(*,*) 'RemoteTag= ', RemoteTag 
	write(*,*) 'RemotePass=' 
	write(*,'(z3,z3,z3,z3,z3,z3,z3,z3)') buffer(1),buffer(2),buffer(3),buffer(4),buffer(5),buffer(6),buffer(7),buffer(8)
		
end

	