! ==========================================================================
! Demo: Secure File Transfe
! Caution: no error check for the sake of briefness and understandability
! Tips:
! The ROCKEY5 dongle supports Secure File Transfer (SFT). The Remote Update Management (RUM) 
! function will verify the user's identity, but it cannot update the dongle algorithm because the 
! end user does not have access to the super password which is required to write to the dongle. 
! The SFT process allows the software developer to securely update dongle algorithms or data files 
! without compromising the super password.
! See the SFT operational overview described below:
! 1. The software developer generates the "cryptography file" by using with own dongle.
! 2. Transfer the file to the user;
! 3. The user transfers the cryptography file to dongle with the program provided by the software 
!    developer. The dongle converts the file to clear text file internally.
! We adopts the following security policies:
! 1. The algorithms of cryptography files and clear text files are completely confidential, not 
!    open to anyone;
! 2. The generation of cryptography files and clear text files is 100% performed inside dongle, 
!    no other external work is involved;
! 3. The encryption / decryption key is related to dongle super password and the management code. 
!    It is not necessary to worry that some one else could create a cryptography file valid in the 
!    end user's dongle
! 4. The content of the cryptography file can not be modified, the file ID, its attribute, and 
!    others are part of the cryptography file, and they are all protected by the file check 
!    mechanism.
! Attention::
! 1. The software developer can only generate the cryptography file after super password 
!    verification. The verified super password is part of the encryption key. That is to say, 
!    the super password of the end user should be the same with the super password which the 
!    software developer uses to generate the cryptography file.
! 2. During the cryptography file generation process, a temporary file with the same name will be 
!    created in the software developer's dongle, because the IC card can transfer at most data of 
!    250 bytes once, and many files are larger than that size. Therefore, dongle should have enough 
!    space for the temporary file generation. The file will not be deleted automatically after the 
!    "cryptography file" generation call and the return of the content; you need to delete it 
!    manually.
! ==========================================================================
! Here two security files will be created: an executable file and a data file. The function 
! of the executable file is to read and return the content of the data file. 

!
! Executable file readdata.c, input parameters are the internal file IDs of dongle to be accessed
! Output parameters are the contents of the internal file of dongle.
!
!void main(void)
!{
! word fileID;
! word fileSize;
! byte b_fileSize;
! byte xdata out[30];
! byte xdata result[100],ret;

! get_input(&fileID,0,1,1);
! open_file(&fileID, 0);
! get_file_infor(out, 1);
! fileSize = (out[5]<<8)|out[4];
! read_file(0,result,fileSize);
! b_fileSize=(unsigned char)fileSize;

!set_response(b_fileSize, result);
! exit();
!}
! ==========================================================================

!DEC$FREEFORM

INCLUDE 'head.F90'

integer*4 errcode 			! error code 
integer*4 num 				! number of founded card reader(s)
integer*4 hic 				! handle of Rockey5 device.
integer*4 i ,j
integer*4 ret_code 
CHARACTER*256 reader_name
CHARACTER cmddata(512) 
CHARACTER buffer(256) 	
CHARACTER*256 buffer2
CHARACTER*20  p_null
CHARACTER c_temp(20) 
	
! convert from  readdata.bin 
integer*4,parameter :: READDATA_SIZE = 276
CHARACTER  g_progReadData(276)
data g_progReadData /#02,#01,#04,#75,#0c,#00,#75,#0d,#00,#75,#0e,#08,#e4,#f5,#0f,#75,&
#10,#01,#75,#11,#01,#12,#00,#a0,#75,#0c,#00,#75,#0d,#00,#75,#0e,&
#08,#e4,#f5,#0f,#12,#00,#bf,#75,#0c,#01,#75,#0d,#00,#75,#0e,#00,&
#75,#0f,#01,#12,#00,#ed,#90,#00,#04,#e0,#fe,#a3,#e0,#fd,#ee,#f5,&
#0a,#ed,#f5,#0b,#e4,#f5,#0c,#f5,#0d,#75,#0e,#01,#75,#0f,#00,#75,&
#10,#0c,#85,#0a,#11,#85,#0b,#12,#12,#00,#7d,#d3,#e5,#0b,#94,#ff,&
#e5,#0a,#94,#00,#40,#04,#7f,#ff,#80,#02,#af,#0b,#8f,#0c,#75,#0d,&
#01,#75,#0e,#00,#75,#0f,#0c,#12,#00,#d6,#02,#01,#10,#12,#ff,#00,&
#e5,#0c,#24,#00,#e5,#0d,#24,#01,#e5,#0e,#24,#02,#e5,#0f,#24,#03,&
#e5,#10,#24,#04,#e5,#11,#24,#05,#e5,#12,#24,#06,#12,#ff,#03,#22,&
#12,#ff,#00,#e5,#0c,#24,#00,#e5,#0d,#24,#01,#e5,#0e,#24,#02,#e5,&
#0f,#24,#03,#e5,#10,#24,#04,#e5,#11,#24,#05,#12,#ff,#0b,#22,#12,&
#ff,#00,#e5,#0c,#24,#00,#e5,#0d,#24,#01,#e5,#0e,#24,#02,#e5,#0f,&
#24,#03,#12,#ff,#02,#22,#12,#ff,#00,#e5,#0d,#24,#00,#e5,#0e,#24,&
#01,#e5,#0f,#24,#02,#e5,#0c,#24,#03,#12,#ff,#0a,#22,#12,#ff,#00,&
#e5,#0c,#24,#00,#e5,#0d,#24,#01,#e5,#0e,#24,#02,#e5,#0f,#24,#03,&
#12,#ff,#06,#22,#78,#7f,#e4,#f6,#d8,#fd,#75,#81,#12,#02,#00,#03,&
#12,#ff,#0d,#22/

integer*4 exesize, datasize, readsize
character	c_exefile(512), c_datafile(512)


	num = DIC_Find() 
	do i=0 ,num-1
		hic = DIC_Open(i, loc(reader_name)) 
		if (hic >= 0) then
		exit
		endif		! Stop searching once one is found.
	enddo

	if (i == num)  then
        	stop
	endif

	! a. Change to super user status (Only super users are allowed to create executable files)
	do i=1 ,8
	cmddata(i)=char(255)  ! Super password is 8 FFs for demo ROCKEY
	enddo		
	errcode = DIC_Command(hic, CHECK_SUPER_PASS, loc(cmddata)) 
	
	! b. Format ROCKEY, all files and directorys will be cleared.
	!    You can set ATR and VOLUME here.
	buffer2 ='DICSYSTEM'C
	ret_code=DIC_Set(loc(cmddata), VOLUME, BY_ARRAY, 0, loc(buffer2)) 
	buffer2 ='DIC Co.'c
	ret_code=DIC_Set(loc(cmddata), ATR, BY_ARRAY, 0,loc(buffer2) ) 
	errcode = DIC_Command(hic, FORMAT_CARD, loc(cmddata)) 

	! c. Create the cryptography file of the executable file
	!    Difference between creating cryptography file and common file is that after setting the
	!    relevant file attributes FILE_DATA call is needed to transfer the file content.
	ret_code=DIC_Set(loc(cmddata), FILL, 512, 0, loc(p_null)) 					! Clear buffer
	ret_code=DIC_Set(loc(cmddata), FILE_ID, BY_VALUE, #0023, loc(p_null)) 				! file ID
	ret_code=DIC_Set(loc(cmddata), FILE_CLASS, BY_VALUE, #ff, loc(p_null)) 				! file class
	ret_code=DIC_Set(loc(cmddata), FILE_ATTRIBUTE, BY_VALUE, FILEATTR_EXEC, loc(p_null)) 		! file attr
	ret_code=DIC_Set(loc(cmddata), FILE_SIZE, BY_VALUE, READDATA_SIZE, loc(p_null)) 		! file size
    data buffer /'6','0','8','8'/
	ret_code=DIC_Set(loc(cmddata), FILE_NAME, BY_ARRAY, 0, loc(buffer)) 				! exe file name
	ret_code=DIC_Set(loc(cmddata), FILE_DATA, IOR(BY_ARRAY , READDATA_SIZE), 0, loc(g_progReadData))! file content
	errcode = DIC_Command(hic, CRYPTOTEXT_FILE, loc(cmddata)) 
	exesize = DIC_Get(loc(cmddata), FILE_DATA, BY_ARRAY, loc(c_exefile)) 

	! d. Create the cryptography file of the data file, data content is 0-9 repeated 8 times.
	ret_code=DIC_Set(loc(cmddata), FILL, 256, 0, loc(p_null)) 				! Clear buffer
	ret_code=DIC_Set(loc(cmddata), FILE_ID, BY_VALUE, #0057, loc(p_null)) 			! file ID
	ret_code=DIC_Set(loc(cmddata), FILE_CLASS, BY_VALUE, #ff, loc(p_null)) 			! file class
	ret_code=DIC_Set(loc(cmddata), FILE_ATTRIBUTE, BY_VALUE, FILEATTR_NORMAL, loc(p_null)) 	! file attr
	ret_code=DIC_Set(loc(cmddata), FILE_SIZE, BY_VALUE, 80, loc(p_null)) 			! file size
	buffer2 = 'File1'c
	ret_code=DIC_Set(loc(cmddata), FILE_NAME, BY_ARRAY, 0, loc(buffer2)) 			! file name
	do i=0,7
	  do j=1 ,10
	  buffer(i*10 +j)= char(#30+ j-1)
	  enddo
	end do
	ret_code=DIC_Set(loc(cmddata), FILE_DATA, IOR(BY_ARRAY , 80), 0, loc(buffer)) 		! file content
	errcode = DIC_Command(hic, CRYPTOTEXT_FILE, loc(cmddata)) 
	datasize = DIC_Get(loc(cmddata), FILE_DATA, BY_ARRAY, loc(c_datafile)) 

	! e. Select the temporary file of the executable cryptography file(#0023)
	ret_code=DIC_Set(loc(cmddata), FILE_ID, BY_VALUE, #0023, loc(p_null)) 
	errcode = DIC_Command(hic, SET_CURRENT_FILE, loc(cmddata)) 

	! f. Delete temporary file of the executable cryptography file(#0023)
	errcode = DIC_Command(hic, REMOVE_FILE, loc(cmddata)) 

	! g. Select the temporary file of the data cryptography file(#0057)
	ret_code=DIC_Set(loc(cmddata), FILE_ID, BY_VALUE, #0057, loc(p_null)) 
	errcode = DIC_Command(hic, SET_CURRENT_FILE, loc(cmddata)) 

	! h. Delete the temporary file of the data cryptography file(#0057)
	errcode = DIC_Command(hic, REMOVE_FILE, loc(cmddata)) 

	! i. Lower the super password verified status to common user status, as it is unnecessary
	!    for super user to create the clear text file.
	errcode = DIC_Close(hic) 
	errcode = DIC_Open(hic, loc(p_null)) 

	! j. Create the clear text file of the executable file
	errcode = DIC_Command(hic, PLAINTEXT_FILE, loc(c_exefile)) 

	! k. Create the clear text file of the data file
    errcode = DIC_Command(hic, PLAINTEXT_FILE, loc(c_datafile)) 

	! l. Run the executable file(#0023), to get the content of the data file(#0057)
	!    and return
	ret_code=DIC_Set(loc(buffer), 0, 2, #0057, loc(p_null)) 		! Input the file ID
	ret_code=DIC_Set(loc(cmddata), RUN_DATA, 2, #6088, loc(buffer)) 	! Set the run data
	errcode = DIC_Command(hic, RUN, loc(cmddata)) 
	readsize = DIC_Get(loc(cmddata), RUN_DATA, BY_ARRAY, loc(buffer)) 	! Fetch data after execution
	do i=0 ,7
	  do j=1,10
	    c_temp(j)=buffer( i * 10 + j)
	  enddo
	  write(*,*)c_temp
	enddo
	
	! m. close
	errcode = DIC_Close(hic) 
end

	