*- ====================================================================

*- ====================================================================

*- 1. Search for dongle device(s) connected to the computer
*-DECLARE integer   DIC_Find IN Dic32U.dll

*- Return value:
*- it is < 0, indicating an error code is returned, for details, please see below.
*- If it is = 0, no card reader device exists
*- If it is >0, the return value is the number of card reader device(s) 

*-(Note: Normally, even if the user does not connect any device, the driver program will report the card presence 
*-due to the existence of the driver. It is necessary to confirm the real device presence with command DIC_Open.) 
*-Suppose that maximum 32 card reader devices can be connected to a computer. 

*- ====================================================================

*- 2. Open the specified card reader
*- DECLARE integer   DIC_Open IN Dic32U.dll integer hic, string @  reader_name

*- Input:
*- hic	Card handle,it's a integer value.
*-(1)positive value, it is a enumeration value,any one of 0, 1, 2, ... return value of DIC_Find(). If reader_name 
*-     is not null, card reader name is returned.
*-(2)negative value,reader_name parameter can not be null,DIC_Open will attempt to open the card reader directly 
*-     by the device name in reader_name; in case of a virtual device, it is the path directory of the file to be opened. 

*- reader_name  name of card reader.
*- An input/output parameter, see comments on hic

*- Return value:
*- >=0				Success, the opened handle is returned
*- <0				An error code is returned, for detailed descriptions, please refer to code defined below.
*- ====================================================================

*- 3. Close the specified card reader

*- DECLARE integer   DIC_Close  IN Dic32U.dll  integer   hic

*- Input:
*- hic: Card handle, must be consistent with the hic returned by DIC_Open. 
*- Return:
*- An error code , for detailed descriptions, please refer to code defined below.

*- ====================================================================

*- 4. Create virtual IC card file(*virtual card private instruction)
*- DECLARE integer   DIC_Create  IN Dic32U.dll  string @  filename, string @  mfname, string  atr, long atrlen
*- Input:
*- filename:	Path file name to be created 
*- mfname:		Main volume name
*- atr:			Software developer information 
*- atrlen:		Software developer information length

*- Return :
*- An error code , for detailed descriptions, please refer to code defined below.

*- ======================================================================

*- 5. Get the dynamic library version
*- DECLARE integer   DIC_GetVersion  IN Dic32U.dll  string @  ver
*- Input:
*- ver: returns the version number of the dynamic library, the version content is:
*-    [V/R]xx.yy
*- For example, R01.10 means it is "Real card dynamic library Version 1.10 "
*-              R03.02 indicats "Virtual card dynamic library Version 3.02 "

*- Return:
*-	An error code , for detailed descriptions, please refer to code defined below.

*- ======================================================================

*- 6. Complete operations on the specified device 

*- DECLARE integer   DIC_Command  IN Dic32U.dll  integer  hic, integer cmd, string @  cmddata

*- Input
*- hic		Device handle 
*- cmd		Specific operation, different constants represent different operations, for details, see below.
*- cmddata	Input/output data related to the command,see comments on below.

*- Command defination
#define GET_CARD_INFO				0x02000000
#define GET_HARDWARE_INFO			0x00FF1C00
#define GET_MANAGER_CODE			0x00FF1A00
#define GET_CARD_PRIVILEGE			0x00FF2200
#define GET_REMOTE_INFO				0x00FF1e00
#define SET_REMOTE_INFO				0x00FF1003
#define CHECK_REMOTE_INFO			0x01FF1003
#define GET_CURRENT_DIR				0x00001800
#define SET_CURRENT_DIR				0x0000a400
#define GET_CURRENT_FILE			0x00001801
#define SET_CURRENT_FILE			0x0100a400
#define GET_PARENT_DIR				0x00001802
#define SET_PARENT_DIR				0x0000a402
#define LIST_DIR				0x0000b200
#define	READ_FILE				0x0000b000
#define WRITE_FILE				0x0000d000
#define FORMAT_CARD				0x0000e000
#define CREATE_DIR				0x0000e001
#define CREATE_FILE				0x0100e001
#define REMOVE_DIR				0x0000ee00
#define REMOVE_FILE				0x0000ee01
#define RANDOM					0x00008400
#define RUN					0x03000000
#define CRYPTOTEXT_FILE				0x04000000
#define PLAINTEXT_FILE				0x05000000

#define CHECK_SUPER_PASS			0x00FF1002
#define SET_SUPER_PASS				0x00FFF402
#define GET_UPGRADE_REMOTE_PASS			0x00FF2000

#define DESENC					0x00005000
#define DESDEC					0x00005100
#define RSAGENKEY				0x00003300
#define RSAENC					0x00003400
#define RSADEC					0x00003500

#define GETFREESPACE				0X00003000

#define SETCOUNTER				0x00005700
#define STEPCOUNTER				0x00005800
*- data for GET_CARD_PRIVILEGE only contain one byte.
*- use those bit value to show current privilege status:
#define DICPR_NOPASS		0
#define	DICPR_SUPERPASS		4
#define	DICPR_REMOTEPASS	8


*- Definitions of diratrpri(directory attribute) and fileatrpri(file attribute)
*- Normal data file
#define FILEATTR_NORMAL			0x00	
*- Executable  
#define FILEATTR_EXEC			0x10	
*- Directory
#define FILEATTR_DIR			0x20	
*- If the current security level is already higher than that of the executable -
*- program, the program cannot be executed. 
#define FILEATTR_UPIGNORE		0x40	
*- File used by internal program and can not be operated externally. 
*- If an executable file is marked as internal,it has hiding property,such files can 
*- be viewed when listing directory only if the super password is verified.
#define FILEATTR_INTERNAL		0x80		

*- Note: the return of LIST_DIR might be either DICST_Dir or DICST_File,
*- you can check it by FILEATTR_DIR

*- ====================================================================

*- 7. Get data from return value of DIC_Cmd
*- DECLARE integer   DIC_Get  IN Dic32.dll  string @  xdata, integer  p1, integer   p2, string @  buffer

*- Input:
*- xdata	Source data buffer
*- p1		Mode/shift, if the highest bit is 1,it is a mode operation,otherwise, it is a user defined shift value
*- p2		Return mode / size, the highest 2 bits represent return mode, others indicate the operation size
*- buffer	Character type buffer where the data is got

*- ====================================================================

*- 8. Set data for DIC_Cmd
*- DECLARE integer   DIC_Set  IN Dic32.dll  string @  xdata, integer  p1, integer  p2, integer   p3, string @  buffer
*- Input:
*- xdata	Targeted buffer
*- p1		Mode/shift, if the highest bit is 1, it is a mode operation,otherwise, it is user defined shift value
*- p2		Transfer mode/ size, the highest 2 bits specify the transfer mode, others indicate the operation size
*- p3		Normally the value that the user needs to store
*- buffer	Character type buffer of the source data, it can be NULL

*- ====================================================================
*- For the p1 parameter of DIC_Get and DIC_Set, some macros are defined to specify the operation objects. 
*- The highest bit of the macros is set to 1. The following are the pre-defined macros: 
#define FILL				    	0x80000000
#define ATR				        0x80010000
#define VOLUME				    	0x80020000
#define FACTORY_TIME				0x80030000
#define HARD_SERIAL				0x80040000
#define SHIP_TIME				0x80050000
#define COS_VERSION				0x80060000
#define ZONE					0x80070000
#define AGENT					0x80080000
#define USER1					0x80090000
#define USER2					0x800A0000
#define DIR_ID					0x800B0000
#define DIR_CLASS				0x800C0000
#define DIR_ATTRIBUTE				0x800D0000
#define DIR_PRIVILEGE				0x800E0000
#define FILE_NO					0x800F0000
#define DIR_NAME				0x80100000
#define FILE_ID					0x80110000
#define FILE_CLASS				0x80120000
#define FILE_ATTRIBUTE				0x80130000
#define FILE_PRIVILEGE				0x80140000
#define FILE_SIZE				0x80150000
#define FILE_NAME				0x80160000
#define WRITE_DATA				0x80170000
#define READ_DATA				0x80180000
#define RUN_DATA				0x80190000
#define RANDOM_SIZE				0x801A0000
#define REMOTE_TAG				0x801B0000
#define REMOTE_PASS				0x801C0000
#define UPGRADE_HARD_SERIAL			0x801D0000
#define UPGRADE_REMOTE_TAG			0x801E0000
#define UPGRADE_REMOTE_PASS			0x801F0000
#define REMOTEPASS_STATUS			0x80200000
#define SUPERPASS_STATUS			0x80210000
#define SUPERPASS_DATA				0x80220000
#define FILE_DATA				0x80230000
#define DES_KEYFILEID				0x80240000
#define DES_KEYINDEX				0x80250000
#define DES_DATA				0x80260000

*DICST_After_EncDec_Data	//Used by DIC_Get
#define AFTER_ENCDEC_DATA			0x80270000

*DICST_Rsa_Data			//Used by DIC_Set
#define RSA_KEYFILEID				0x80280000
#define RSA_DATA				0x80290000

*DICST_Rsa_GenKey			//Used by Dic_Set
#define RSA_PUBKEYID				0x802A0000
#define RSA_PUBKEYSIZE				0x802B0000
#define RSA_PIVKEYID				0x802C0000

*- ====================================================================
*-For the p2 parameter of DIC_Get and DIC_Set, the highest 2 bits indicate data transfer or return mode, 
#define BY_VALUE				0x00000000
#define BY_DOUBLE				0x40000000
#define BY_ARRAY				0x80000000
#define BY_STRING				0xC0000000

*- for  DIC_GetData
*- BY_VALUE		returned in return value
*- BY_ARRAY		returned in pstr 
*- BY_STRING		returned in pstr 

*- for  DIC_SetData
*- BY_VALUE		set data via p3 
*- BY_ARRAY		set data via pstr 
*- BY_STRING		set data via pstr 
*- ====================================================================

*- Error code ===========================================================

#ifndef SCARD_S_SUCCESS

*- Success
#define SCARD_S_SUCCESS 			0x00000000 

*- Internal link check failure
#define SCARD_F_INTERNAL_ERROR			 0x80100001 

*- Operation is terminated by user
#define SCARD_E_CANCELLED			0x80100002 

*- Incorrect operation handle
#define SCARD_E_INVALID_HANDLE			 0x80100003 

*- Incorrect parameter
#define SCARD_E_INVALID_PARAMETER		 0x80100004 

*- Registered startup information lost or invalid 
#define SCARD_E_INVALID_TARGET			 0x80100005 

*- Memory not enough for command completion 
#define SCARD_E_NO_MEMORY			0x80100006 

*- Internal overtime
#define SCARD_F_WAITED_TOO_LONG			 0x80100007 

*- User specified buffer is too small, not enough for the return data
#define SCARD_E_INSUFFICIENT_BUFFER		 0x80100008 

*- Unknown card reader
#define SCARD_E_UNKNOWN_READER			 0x80100009 

*- Over user specified time
#define SCARD_E_TIMEOUT				0x8010000A 

*- Card occupied by other connection
#define SCARD_E_SHARING_VIOLATION		 0x8010000B 

*- No card in the card reader
#define SCARD_E_NO_SMARTCARD			 0x8010000C 

*- Unknown card type
#define SCARD_E_UNKNOWN_CARD			 0x8010000D 

*- Card reader can not complete card eject operation
#define SCARD_E_CANT_DISPOSE			 0x8010000E 

*- Current card does not support user specified communication protocol
#define SCARD_E_PROTO_MISMATCH			 0x8010000F 

*- Card is not ready for receiving commands
#define SCARD_E_NOT_READY			0x80100010 

*- Some variable values are not suitable
#define SCARD_E_INVALID_VALUE			 0x80100011 

*- Operation is terminated by system, it may be necessary to log on again or shut down. 
#define SCARD_E_SYSTEM_CANCELLED		 0x80100012 

*- Internal communication error
#define SCARD_F_COMM_ERROR			0x80100013 

*- Unknown internal error
#define SCARD_F_UNKNOWN_ERROR			 0x80100014 

*- Invalid vendor information
#define SCARD_E_INVALID_ATR			0x80100015 

*- User attempts to terminate some process not available
#define SCARD_E_NOT_TRANSACTED			 0x80100016 

*- The specifiied card reader is not ready
#define SCARD_E_READER_UNAVAILABLE		 0x80100017 

*- Operation is terminated, service program is allowed to exit
#define SCARD_P_SHUTDOWN			0x80100018 

*- PCI receiving buffer is too small
#define SCARD_E_PCI_TOO_SMALL			 0x80100019 

*- Card reader drive does not support the current card reader
#define SCARD_E_READER_UNSUPPORTED		 0x8010001A 

*- Card reader driver program can not create an exclusive name, card reader with the same name already exists
#define SCARD_E_DUPLICATE_READER		 0x8010001B 

*- Card is not supported by the current card reader
#define SCARD_E_CARD_UNSUPPORTED		 0x8010001C 

*- Smart card service is not enabled
#define SCARD_E_NO_SERVICE			0x8010001D 

*- Smart card service is disabled
#define SCARD_E_SERVICE_STOPPED			 0x8010001E 

*- Some unexpected smart card error
#define SCARD_E_UNEXPECTED			 0x8010001F 

*- Smart card software developer information not accessible
#define SCARD_E_ICC_INSTALLATION		 0x80100020 

*- Smart card vendor information not accessible
#define SCARD_E_ICC_CREATEORDER			 0x80100021 

*- Current smart card does not support the function user demanded
#define SCARD_E_UNSUPPORTED_FEATURE		 0x80100022 

*- The specified directory does not exist
#define SCARD_E_DIR_NOT_FOUND			 0x80100023 

*- The specified file does not exist
#define SCARD_E_FILE_NOT_FOUND			 0x80100024 

*- The specified directory is no longer a valid directory
#define SCARD_E_NO_DIR				0x80100025 

*- The specified file is no longer a valid file, no file selected
#define SCARD_E_NO_FILE				0x80100026 

*- The file denies any access
#define SCARD_E_NO_ACCESS			0x80100027 

*- Card is full, no more data write
#define SCARD_E_WRITE_TOO_MANY			 0x80100028 

*- File pointer error
#define SCARD_E_BAD_SEEK			0x80100029 

*- PIN code error
#define SCARD_E_INVALID_CHV			0x8010002A 

*- An unrecognized error code is returned from smart card service
#define SCARD_E_UNKNOWN_RES_MNG			 0x8010002B 

*- The certificate requested does not exist
#define SCARD_E_NO_SUCH_CERTIFICATE		 0x8010002C 

*- The certificate requested is not available
#define SCARD_E_CERTIFICATE_UNAVAILABLE	 	0x8010002D 

*- No card reader found
#define SCARD_E_NO_READERS_AVAILABLE	 	0x8010002E 

*- Data loss during smart card communication, please try again
#define SCARD_E_COMM_DATA_LOST			 0x8010002F 

*- The key file requested does not exist
#define SCARD_E_NO_KEY_CONTAINER		 0x80100030 

*- Card reader can not communicate with the card due to software developer information conflict
#define SCARD_W_UNSUPPORTED_CARD		 0x80100065 

*- Card does not respond to reset
#define SCARD_W_UNRESPONSIVE_CARD		 0x80100066 

*- Card no power
#define SCARD_W_UNPOWERED_CARD			 0x80100067 

*- Card is reset, shared information is invalid
#define SCARD_W_RESET_CARD			0x80100068 

*- Card is removed
#define SCARD_W_REMOVED_CARD			 0x80100069 

*- Access is denied due to security rules
#define SCARD_W_SECURITY_VIOLATION		 0x8010006A 

*- PIN code is not verified, access is denied
#define SCARD_W_WRONG_CHV			0x8010006B 

*- Max. PIN code retry number is reached, access is denied
#define SCARD_W_CHV_BLOCKED			0x8010006C 

*- The last smart card file is reached, no more file available
#define SCARD_W_EOF				0x8010006D 

*- Operation is terminated by user
#define SCARD_W_CANCELLED_BY_USER		 0x8010006E 

*- Smart card PIN is not set
#define SCARD_W_CARD_NOT_AUTHENTICATED	 	0x8010006F 




*- Extended error code  =========================================================

*- File already exists
#define SCARD_E_FILE_EXISTS				 0xA0100001 

*- Card-resident memory operation error
#define SCARD_E_EPROM_ERROR				 0xA0100002 

*- User specified CLA is invalid
#define SCARD_E_INVALID_CLA				 0xA0100003 

*- User specified INS is invalid
#define SCARD_E_INVALID_INS				 0xA0100004 

*- Virtual machine address out ranged/abnormal 
#define SCARD_E_VM_ADDRESS_ERROR		 0xA0100005 

*- 0 division error
#define SCARD_E_ZERO_DIVIDE				 0xA0100006 

*- Card is not inserted to the proper place
#define SCARD_E_WRONG_POSITION			 0xA0100007 

*- Card is in an unknown status
#define SCARD_E_UNKNOWN_STATE			 0xA0100008 

*- Card is not opened
#define SCARD_E_CARD_NOT_OPENED			 0xA0100009 

*- Unknown command
#define SCARD_E_UNKNOWN_COMMAND			 0xA010000A 

*- Super password reset number is 0
#define SCARD_E_ZERO_TRYTIME			 0xA010000B 

*- Too many devices are opened
#define SCARD_E_TOO_MANY_DEVICE			 0xA010000C 

*- Invalid instruction.
#define	SCARD_E_INVALID_INSTRUCTION		 0xA010000D 

*- card still have more data to return 
#define SCARD_W_RESPONSE			0xA01000FF 


*- Error codes exclusive for virtual devices=================================================

*- Virtual card file creation failure
#define SCARD_E_FILE_CREATE_FAILED		 0xA0101001 

*- Virtual card file open failure
#define SCARD_E_FILE_OPEN_FAILED		 0xA0101002 
