// ==========================================================================
// RSA encryption/decryption
// Description: 
// If DES_RSA library is loaded, the DES, 3DES and RSA can be implemented by the following steps:
// (1) creat RSA key pair
// (2) encrypt/decrypt based on public key file ID or private key file ID
// Note:
// (1) the unit of key is bit, only support 1024 or 512 bits key
// (2) the default value of e is 65537, and cannot be changed
// (3) when encrypt data, if the data is longer than n, please set the high bit of data to 0
// ==========================================================================

import java.io.*;

public class Sample15
{
	public Sample15()
	{
	}
	public static void main(String [] args)
	{
		int i, num, hic=0, errcode;
		byte []cmddata=new byte[256],dataSrc=new byte[128];
	    
		byte []EncData=new byte[128];
		int wRetLen;
	
		JavaRockey Rockey=new JavaRockey();
		System.out.print("Searching ROCKEEY ");
		num = Rockey.DIC_Find();
		if (num <= 0)
		{
			System.out.println("No ROCKEY found");
			return;
		}
		System.out.println(num);
	
		System.out.println("Open ROCKEY");
		for (i=0;i<num;i++)
		{
			hic = Rockey.DIC_Open(i, null);
			if (hic >= 0) break;		// find the first ROCKEY
		}
		if (i == num)
		{
			System.out.println("No ROCKEY found");
			return;
		}
	
		for(i=0;i<8;i++) cmddata[i]=(byte)0xff;
		errcode = Rockey.DIC_Command(hic, Rockey.CHECK_SUPER_PASS, cmddata);
		if (errcode != Rockey.SCARD_S_SUCCESS) {
			System.out.println("Verify Super Pass error : "+Integer.toHexString(errcode));
		}
	    
	
		//////////////////////////////////////////////////////////////////////////
		//generate RSA key pair
		Rockey.DIC_Set(cmddata, Rockey.RSA_PUBKEYID, Rockey.BY_VALUE, 0x7f01, null); //public key file ID
		Rockey.DIC_Set(cmddata, Rockey.RSA_PUBKEYSIZE, Rockey.BY_VALUE, 1024, null); //public key size in bit
		Rockey.DIC_Set(cmddata, Rockey.RSA_PIVKEYID, Rockey.BY_VALUE, 0x7f02, null); //private key file ID
		errcode = Rockey.DIC_Command(hic, Rockey.RSAGENKEY, cmddata);
		if (errcode != Rockey.SCARD_S_SUCCESS) {
			System.out.println("RSA key error: "+Integer.toHexString(errcode));
		}
	
		//set plaintext
		for(i = 0; i<127;i++){
			dataSrc[i]=(byte)i;
		}
		dataSrc[124]= 0; 			 //set high byte 0
		
		//RSA encryption, either public key or private key
		Rockey.DIC_Set(cmddata, Rockey.RSA_KEYFILEID, Rockey.BY_VALUE, 0x7f01, null); //use public key here
		Rockey.DIC_Set(cmddata, Rockey.RSA_DATA, Rockey.BY_ARRAY | 128, 0, dataSrc);
		errcode = Rockey.DIC_Command(hic, Rockey.RSAENC, cmddata);
		if (errcode != Rockey.SCARD_S_SUCCESS) 
		{
			System.out.println("RSA enc error : "+Integer.toHexString(errcode));
		}else
		{
			wRetLen = Rockey.DIC_Get(cmddata, Rockey.AFTER_ENCDEC_DATA, Rockey.BY_ARRAY, EncData);
			System.out.println("Get "+wRetLen+" Cipher text:");
			for(i= 0;i<wRetLen;i++){
				System.out.print(Integer.toHexString(EncData[i]&0xff)+" ");
				if((i+1)%16 == 0) System.out.println("");
			}
		}
		
		//RSA decrypt
		Rockey.DIC_Set(cmddata, Rockey.RSA_KEYFILEID, Rockey.BY_VALUE, 0x7f02, null);
		Rockey.DIC_Set(cmddata, Rockey.RSA_DATA, Rockey.BY_ARRAY | 128, 0, EncData);
		errcode = Rockey.DIC_Command(hic, Rockey.RSADEC, cmddata);   
		if (errcode != Rockey.SCARD_S_SUCCESS) 
		{
			System.out.println("RSA dec error : "+Integer.toHexString(errcode));
		}else
		{
			wRetLen = Rockey.DIC_Get(cmddata, Rockey.AFTER_ENCDEC_DATA, Rockey.BY_ARRAY, EncData);
			System.out.println("Get "+wRetLen+" plant text:");
			for(i= 0;i<wRetLen;i++){
				System.out.print(Integer.toHexString(EncData[i]&0xff)+" ");
				if((i+1)%16 == 0) System.out.println("");
			}
		}
			
		Rockey.DIC_Close(hic);
		}

}