Attribute VB_Name = "Module1"
' ====================================================================
' 0. Search for dongle device(s) connected to the computer
Declare Function DIC_Find Lib "Dic32u.DLL" () As Long

' Return value:
' it is < 0, indicating an error code is returned, for details, please see below.
' If it is = 0, no card reader device exists
' If it is >0, the return value is the number of card reader device(s) 

'(Note: Normally, even if the user does not connect any device, the driver program will report the card presence 
'due to the existence of the driver. It is necessary to confirm the real device presence with command DIC_Open.) 
'Suppose that maximum 32 card reader devices can be connected to a computer. 


' ====================================================================
' 1. Find a dongle that has specific management code 
Declare Function DIC_FindByMgrCode Lib "Dic32u.DLL" (mgrcode As Any) As Long
' Input:
' pMgrCode	a pointer to DICST_ManagerCode structure
' 
' Return:
' The same as DIC_Find

' ====================================================================
' 2. Open the specified card reader
Declare Function DIC_Open Lib "Dic32u.DLL" (ByVal hic As Long, ByVal reader_name As String) As Long

' Input:
' hic	Card handle,it's a integer value.
' (1)positive value, it is a enumeration valueany one of 0, 1, 2, ... return value of DIC_Find(). If reader_name 
'     is not null, card reader name is returned.
' (2)negative valuereader_name parameter can not be nullDIC_Open will attempt to open the card reader directly 
'     by the device name in reader_name; in case of a virtual device, it is the path directory of the file to be opened. 

' reader_name  name of card reader.
' An input/output parameter, see comments on hic

' Return value:
' >=0				Success, the opened handle is returned
' <0				An error code is returned, for detailed descriptions, please refer to code defined below.
' ====================================================================

' 3. Close the specified card reader
Declare Function DIC_Close Lib "Dic32u.DLL" (ByVal hic As Long) As Long

' Input:
' hic: Card handle, must be consistent with the hic returned by DIC_Open. 
' Return:
' An error code , for detailed descriptions, please refer to code defined below.

' ====================================================================

' 4. Create virtual IC card file(*virtual card private instruction)
Declare Function DIC_Create Lib "Dic32u.DLL" (ByVal filename As String, ByVal mfname As String, atrbuffer As Any, ByVal atrlen As Integer) As Long

' Input:
' filename:	Path file name to be created 
' mfname:		Main volume name
' atr:			Software developer information 
' atrlen:		Software developer information length

' Return :
' An error code , for detailed descriptions, please refer to code defined below.

' ======================================================================

' 5. Get the dynamic library version
Declare Function DIC_GetVersion Lib "Dic32u.DLL" (ByVal ver As String) As Long

' Input:
' ver: returns the version number of the dynamic library, the version content is:
'    [V/R]xx.yy
' For example, R01.10 means it is "Real card dynamic library Version 1.10 "
'              R03.02 indicats "Virtual card dynamic library Version 3.02 "

' Return:
'	An error code , for detailed descriptions, please refer to code defined below.

' ======================================================================

' 6. Complete operations on the specified device 
Declare Function DIC_Command Lib "Dic32u.DLL" (ByVal hic As Long, ByVal cmd As Long, cmddata As Any) As Long
' Input
' hic		Device handle 
' cmd		Specific operation, different constants represent different operations, for details, see below.
' cmddata	Input/output data related to the commandsee comments on below.

' Command defination
Global Const GET_CARD_INFO As Long = &H2000000
Global Const GET_HARDWARE_INFO As Long = &HFF1C00
Global Const GET_MANAGER_CODE As Long = &HFF1A00
Global Const GET_CARD_PRIVILEGE As Long = &HFF2200
Global Const GET_REMOTE_INFO As Long = &HFF1E00
Global Const SET_REMOTE_INFO As Long = &HFF1003
Global Const CHECK_REMOTE_INFO As Long = &H1FF1003
Global Const GET_CURRENT_DIR As Long = &H1800
Global Const SET_CURRENT_DIR As Long = 41984 '&HA400
Global Const GET_CURRENT_FILE As Long = &H1801
Global Const SET_CURRENT_FILE As Long = &H100A400
Global Const GET_PARENT_DIR As Long = &H1802
Global Const SET_PARENT_DIR As Long = 41986 '&HA402
Global Const LIST_DIR As Long = 45568 '&HB200
Global Const READ_FILE As Long = 45056 '&HB000
Global Const WRITE_FILE As Long = 53248 '&HD000
Global Const FORMAT_CARD As Long = 57344 '&HE000
Global Const CREATE_DIR As Long = 57345 '&HE001
Global Const CREATE_FILE As Long = &H100E001
Global Const REMOVE_DIR As Long = 60928 '&HEE00
Global Const REMOVE_FILE As Long = 60929 '&HEE01
Global Const RANDOM As Long = 33792 '&H8400
Global Const RUN As Long = &H3000000
Global Const CRYPTOTEXT_FILE As Long = &H4000000
Global Const PLAINTEXT_FILE As Long = &H5000000

Global Const CHECK_SUPER_PASS As Long = &HFF1002
Global Const SET_SUPER_PASS As Long = &HFFF402
Global Const GET_UPGRADE_REMOTE_PASS As Long = &HFF2000

Global Const DESENC                       As Long = &H5000
Global Const DESDEC                       As Long = &H5100
Global Const RSAGENKEY                    As Long = &H3300
Global Const RSAENC                       As Long = &H3400
Global Const RSADEC                       As Long = &H3500

Global Const GETFREESPACE                 As Long = &H3000

Global Const STARTCOUNTER                 As Long = &H5700
Global Const SETCOUNTER                 As Long = &H5700
Global Const STEPCOUNTER                  As Long = &H5800

' Definitions of diratrpri(directory attribute) and fileatrpri(file attribute)
' Normal data file
Global Const FILEATTR_NORMAL = &H0
' Executable  
Global Const FILEATTR_EXEC = &H10
' Directory
Global Const FILEATTR_DIR = &H20
' If the current security level is already higher than that of the executable program, the program cannot be executed. 
Global Const FILEATTR_UPIGNORE = &H40
' File used by internal program and can not be operated externally. If an executable file is marked as internalit has hiding propertysuch files can be viewed when listing directory only if the super password is verified.
Global Const FILEATTR_INTERNAL = &H80
' 

' ====================================================================

' 7. Get data from return value of DIC_Cmd
Declare Function DIC_Get Lib "Dic32u.DLL" (xdata As Any, ByVal p1 As Long, ByVal p2 As Long, buffer As Any) As Long

' Input:
' xdata	Source data buffer
' p1		Mode/shift, if the highest bit is 1it is a mode operationotherwise, it is a user defined shift value
' p2		Return mode / size, the highest 2 bits represent return mode, others indicate the operation size
' buffer	Character type buffer where the data is got

' ====================================================================

' 8. Set data for DIC_Cmd
Declare Function DIC_Set Lib "Dic32u.DLL" (xdata As Any, ByVal p1 As Long, ByVal p2 As Long, ByVal p3 As Long, buffer As Any) As Long

' Input:
' xdata	Targeted buffer
' p1		Mode/shift, if the highest bit is 1, it is a mode operationotherwise, it is user defined shift value
' p2		Transfer mode/ size, the highest 2 bits specify the transfer mode, others indicate the operation size
' p3		Normally the value that the user needs to store
' buffer	Character type buffer of the source data, it can be NULL

' ====================================================================
' For the p1 parameter of DIC_Get and DIC_Set, some macros are defined to specify the operation objects. 
' The highest bit of the macros is set to 1. The following are the pre-defined macros: 
Global Const FILL As Long = &H80000000
Global Const ATR As Long = &H80010000
Global Const VOLUME As Long = &H80020000
Global Const FACTORY_TIME As Long = &H80030000
Global Const HARD_SERIAL As Long = &H80040000
Global Const SHIP_TIME As Long = &H80050000
Global Const COS_VERSION As Long = &H80060000
Global Const ZONE As Long = &H80070000
Global Const AGENT As Long = &H80080000
Global Const USER1 As Long = &H80090000
Global Const USER2 As Long = &H800A0000
Global Const DIR_ID As Long = &H800B0000
Global Const DIR_CLASS As Long = &H800C0000
Global Const DIR_ATTRIBUTE As Long = &H800D0000
Global Const DIR_PRIVILEGE As Long = &H800E0000
Global Const FILE_NO As Long = &H800F0000
Global Const DIR_NAME As Long = &H80100000
Global Const FILE_ID As Long = &H80110000
Global Const FILE_CLASS As Long = &H80120000
Global Const FILE_ATTRIBUTE As Long = &H80130000
Global Const FILE_PRIVILEGE As Long = &H80140000
Global Const FILE_SIZE As Long = &H80150000
Global Const FILE_NAME As Long = &H80160000
Global Const WRITE_DATA As Long = &H80170000
Global Const READ_DATA As Long = &H80180000
Global Const RUN_DATA As Long = &H80190000
Global Const RANDOM_SIZE As Long = &H801A0000
Global Const REMOTE_TAG As Long = &H801B0000
Global Const REMOTE_PASS As Long = &H801C0000
Global Const UPGRADE_HARD_SERIAL As Long = &H801D0000
Global Const UPGRADE_REMOTE_TAG As Long = &H801E0000
Global Const UPGRADE_REMOTE_PASS As Long = &H801F0000
Global Const REMOTEPASS_STATUS As Long = &H80200000
Global Const SUPERPASS_STATUS As Long = &H80210000
Global Const SUPERPASS_DATA As Long = &H80220000
Global Const FILE_DATA As Long = &H80230000

'DICST_Des_Data            'Used by DIC_Set
Global Const DES_KEYFILEID            As Long = &H80240000
Global Const DES_KEYINDEX             As Long = &H80250000
Global Const DES_DATA                 As Long = &H80260000

'DICST_After_EncDec_Data   'Used by DIC_Get
Global Const AFTER_ENCDEC_DATA        As Long = &H80270000

'DICST_Rsa_Data            'Used by DIC_Set
Global Const RSA_KEYFILEID            As Long = &H80280000
Global Const RSA_DATA                 As Long = &H80290000

'DICST_Rsa_GenKey          'Used by Dic_Set
Global Const RSA_PUBKEYID             As Long = &H802A0000
Global Const RSA_PUBKEYSIZE           As Long = &H802B0000
Global Const RSA_PIVKEYID             As Long = &H802C0000
' ====================================================================
'For the p2 parameter of DIC_Get and DIC_Set, the highest 2 bits indicate data transfer or return mode, 
Global Const BY_VALUE As Long = &H0
Global Const BY_DOUBLE As Long = &H40000000
Global Const BY_ARRAY As Long = &H80000000
Global Const BY_STRING As Long = &HC0000000

' for  DIC_GetData
' BY_VALUE		returned in return value
' BY_DOUBLE		returned in double in string format, such as "328.002"
' BY_ARRAY		returned in pstr 
' BY_STRING		returned in pstr 

' for  DIC_SetData
' BY_VALUE		set data via p3 
' BY_DOUBLE		set double data in string format, such as "328.002"
' BY_ARRAY		set data via pstr 
' BY_STRING		set data via pstr 
' ====================================================================


' Error code ===========================================================

' Success
Global Const SCARD_S_SUCCESS As Long = &H0

' Internal link check failure
Global Const SCARD_F_INTERNAL_ERROR As Long = &H80100001

' Operation is terminated by user
Global Const SCARD_E_CANCELLED As Long = &H80100002

' Incorrect operation handle
Global Const SCARD_E_INVALID_HANDLE As Long = &H80100003

' Incorrect parameter
Global Const SCARD_E_INVALID_PARAMETER As Long = &H80100004

' Registered startup information lost or invalid 
Global Const SCARD_E_INVALID_TARGET As Long = &H80100005

' Memory not enough for command completion
Global Const SCARD_E_NO_MEMORY As Long = &H80100006

' Internal overtime
Global Const SCARD_F_WAITED_TOO_LONG As Long = &H80100007

' User specified buffer is too small, not enough for the return data
Global Const SCARD_E_INSUFFICIENT_BUFFER As Long = &H80100008

' Unknown card reader
Global Const SCARD_E_UNKNOWN_READER As Long = &H80100009

' Over user specified time
Global Const SCARD_E_TIMEOUT As Long = &H8010000A

' Card occupied by other connection
Global Const SCARD_E_SHARING_VIOLATION As Long = &H8010000B

' No card in the card reader
Global Const SCARD_E_NO_SMARTCARD As Long = &H8010000C

' Unknown card type
Global Const SCARD_E_UNKNOWN_CARD As Long = &H8010000D

' Card reader can not complete card eject operation
Global Const SCARD_E_CANT_DISPOSE As Long = &H8010000E

' Current card does not support user specified communication protocol
Global Const SCARD_E_PROTO_MISMATCH As Long = &H8010000F

' Card is not ready for receiving commands
Global Const SCARD_E_NOT_READY As Long = &H80100010

' Some variable values are not suitable
Global Const SCARD_E_INVALID_VALUE As Long = &H80100011

' Operation is terminated by system, it may be necessary to log on again or shut down.
Global Const SCARD_E_SYSTEM_CANCELLED As Long = &H80100012

' Internal communication error
Global Const SCARD_F_COMM_ERROR As Long = &H80100013

' Unknown internal error
Global Const SCARD_F_UNKNOWN_ERROR As Long = &H80100014

' Invalid vendor information
Global Const SCARD_E_INVALID_ATR As Long = &H80100015

' User attempts to terminate some process not available
Global Const SCARD_E_NOT_TRANSACTED As Long = &H80100016

' The specifiied card reader is not ready
Global Const SCARD_E_READER_UNAVAILABLE As Long = &H80100017

' Operation is terminated, service program is allowed to exit
Global Const SCARD_P_SHUTDOWN As Long = &H80100018

' PCI receiving buffer is too small
Global Const SCARD_E_PCI_TOO_SMALL As Long = &H80100019

' Card reader drive does not support the current card reader
Global Const SCARD_E_READER_UNSUPPORTED As Long = &H8010001A

' Card reader driver program can not create an exclusive name, card reader with the same name already exists
Global Const SCARD_E_DUPLICATE_READER As Long = &H8010001B

' Card is not supported by the current card reader
Global Const SCARD_E_CARD_UNSUPPORTED As Long = &H8010001C

' Smart card service is not enabled
Global Const SCARD_E_NO_SERVICE As Long = &H8010001D

' Smart card service is disabled
Global Const SCARD_E_SERVICE_STOPPED As Long = &H8010001E

' Some unexpected smart card error
Global Const SCARD_E_UNEXPECTED As Long = &H8010001F

' Smart card software developer information not accessible
Global Const SCARD_E_ICC_INSTALLATION As Long = &H80100020

' Smart card vendor information not accessible
Global Const SCARD_E_ICC_CREATEORDER As Long = &H80100021

' Current smart card does not support the function user demanded
Global Const SCARD_E_UNSUPPORTED_FEATURE As Long = &H80100022

' The specified directory does not exist
Global Const SCARD_E_DIR_NOT_FOUND As Long = &H80100023

' The specified file does not exist
Global Const SCARD_E_FILE_NOT_FOUND As Long = &H80100024

' The specified directory is no longer a valid directory
Global Const SCARD_E_NO_DIR As Long = &H80100025

' The specified file is no longer a valid file, no file selected
Global Const SCARD_E_NO_FILE As Long = &H80100026

' The file denies any access
Global Const SCARD_E_NO_ACCESS As Long = &H80100027

' Card is full, no more data write
Global Const SCARD_E_WRITE_TOO_MANY As Long = &H80100028

' File pointer error
Global Const SCARD_E_BAD_SEEK As Long = &H80100029

' PIN code error
Global Const SCARD_E_INVALID_CHV As Long = &H8010002A

' An unrecognized error code is returned from smart card service
Global Const SCARD_E_UNKNOWN_RES_MNG As Long = &H8010002B

' The certificate requested does not exist
Global Const SCARD_E_NO_SUCH_CERTIFICATE As Long = &H8010002C

' The certificate requested is not available
Global Const SCARD_E_CERTIFICATE_UNAVAILABLE As Long = &H8010002D

' No card reader found
Global Const SCARD_E_NO_READERS_AVAILABLE As Long = &H8010002E

' Data loss during smart card communication, please try again
Global Const SCARD_E_COMM_DATA_LOST As Long = &H8010002F

' The key file requested does not exist
Global Const SCARD_E_NO_KEY_CONTAINER As Long = &H80100030

' Card reader can not communicate with the card due to software developer information conflict
Global Const SCARD_W_UNSUPPORTED_CARD As Long = &H80100065

' Card does not respond to reset
Global Const SCARD_W_UNRESPONSIVE_CARD As Long = &H80100066

' Card no power
Global Const SCARD_W_UNPOWERED_CARD As Long = &H80100067

' Card is reset, shared information is invalid
Global Const SCARD_W_RESET_CARD As Long = &H80100068

' Card is removed
Global Const SCARD_W_REMOVED_CARD As Long = &H80100069

' Access is denied due to security rules
Global Const SCARD_W_SECURITY_VIOLATION As Long = &H8010006A

' PIN code is not verified, access is denied
Global Const SCARD_W_WRONG_CHV As Long = &H8010006B

' Max. PIN code retry number is reached, access is denied
Global Const SCARD_W_CHV_BLOCKED As Long = &H8010006C

' The last smart card file is reached, no more file available
Global Const SCARD_W_EOF As Long = &H8010006D

' Operation is terminated by user
Global Const SCARD_W_CANCELLED_BY_USER As Long = &H8010006E

' Smart card PIN is not set
Global Const SCARD_W_CARD_NOT_AUTHENTICATED As Long = &H8010006F


' Extended error code  =========================================================

' File already exists
Global Const SCARD_E_FILE_EXISTS As Long = &HA0100001

' Card-resident memory operation error
Global Const SCARD_E_EPROM_ERROR As Long = &HA0100002

' User specified CLA is invalid
Global Const SCARD_E_INVALID_CLA As Long = &HA0100003

' User specified INS is invalid
Global Const SCARD_E_INVALID_INS As Long = &HA0100004

' Virtual machine address out ranged/abnormal 
Global Const SCARD_E_VM_ADDRESS_ERROR As Long = &HA0100005

' 0 division error
Global Const SCARD_E_ZERO_DIVIDE As Long = &HA0100006

' Card is not inserted to the proper place
Global Const SCARD_E_WRONG_POSITION As Long = &HA0100007

' Card is in an unknown status
Global Const SCARD_E_UNKNOWN_STATE As Long = &HA0100008

' Card is not opened
Global Const SCARD_E_CARD_NOT_OPENED As Long = &HA0100009

' Unknown command
Global Const SCARD_E_UNKNOWN_COMMAND As Long = &HA010000A

' Super password reset number is 0
Global Const SCARD_E_ZERO_TRYTIME As Long = &HA010000B

' Too many devices are opened
Global Const SCARD_E_TOO_MANY_DEVICE As Long = &HA010000C

' Invalid instruction.
Global Const SCARD_E_INVALID_INSTRUCTION As Long = &HA010000D

' card still have more data to return 
Global Const SCARD_W_RESPONSE As Long = &HA01000FF


' Error codes exclusive for virtual devices=================================================

' Virtual card file creation failure
Global Const SCARD_E_FILE_CREATE_FAILED As Long = &HA0101001

' Virtual card file open failure
Global Const SCARD_E_FILE_OPEN_FAILED As Long = &HA0101002

