// ==========================================================================
// DES and 3DES Encryption and Decryption
// Description: 
// If DES_RSA library is loaded, the DES, 3DES and RSA can be implemented by the following steps:
//  (1) creat a internal data file (super password is needed)
//  (2) write DES, RSA key (super password is needed)
//  (3) encrypt/decrypt based on file ID and key ID. 
//	when encrypt, the length of data should be A*8, where A is an integer, otherwise the data will be filled automatically.
// 	when decrypt, please cut off the corresponding data.
// Note:
// (1)DES Key is 8 bytes, and 3DES key is 16 bytes
// (2)The struct for DES and 3DES key
//	DES Key struct (10 bytes): key ID (1 byte)+key length(1 byte, value is 8)+key(8 bytes)
//	3DES Key struct (18 bytes): key ID (1 byte)+key length(1 byte, value is 16)+key(16 byte)
// ==========================================================================
#include <windows.h>
#include <stdio.h>
#include "dic32.h"

void main(int argc, char* argv[])
{
	int i, num, hic, errcode;
	char cmddata[256],outData[40],data[40];
    BYTE DesData[20]={0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99};  //data for des encrypt =8
	WORD wRet,dataLength = 8;
	
	printf("Searching ROCKEY ");
	num = DIC_Find();
	if (num <= 0)
	{
		printf("No ROCKEY found\n");
		return;
	}
	printf("%d\n",num);

	printf("Open ROCKEY\n");
	for (i=0;i<num;i++)
	{
		hic = DIC_Open(i, NULL);	
		if (hic >= 0) break;		// find the first ROCKEY
	}
	if (i == num)
	{
		printf("No ROCKEY found\n");
		return;
	}
	
	//verify super password
	memcpy(cmddata,"\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF",8);		
	errcode = DIC_Command(hic, CHECK_SUPER_PASS, cmddata);
	if (errcode != SCARD_S_SUCCESS) {
		printf("Verify Super Pass error : %8x\n",errcode);
	}

	//format the ROCKEY, all info will be deleted, you can set new ROCKEY info
	DIC_Set(cmddata, VOLUME, BY_ARRAY, 0, "DEFAULT VOL");
	DIC_Set(cmddata, ATR, BY_ARRAY, 0, "DEFAULT ATR");
	errcode = DIC_Command(hic, FORMAT_CARD, cmddata);
	if (errcode != SCARD_S_SUCCESS) {
		printf("Format card error: %8x\n",errcode);
	}


	// create a file for DES or TDES(3DES)
	// file ID $6F01
	DIC_Set(cmddata, FILL, 256, 0, NULL);							// clear buffer
	DIC_Set(cmddata, FILE_ID, BY_VALUE, 0x6F01, NULL);					// file ID
	DIC_Set(cmddata, FILE_CLASS, BY_VALUE, 0xff, NULL);					// file class
	DIC_Set(cmddata, FILE_ATTRIBUTE, BY_VALUE, FILEATTR_NORMAL | FILEATTR_INTERNAL, NULL);	// file attr
	DIC_Set(cmddata, FILE_SIZE, BY_VALUE, 128, NULL);					// file size
	DIC_Set(cmddata, FILE_NAME, BY_ARRAY, 0, "des");					// file name
	errcode = DIC_Command(hic, CREATE_FILE, cmddata);
	if(errcode != SCARD_S_SUCCESS){
		printf("Create Des file error: %8x \n",errcode);
	}


    	// write 8 bytes DES key, note the struct
	memcpy(data,"\x01\x08\x01\x02\x03\x04\x05\x06\x07\x08",10);
	DIC_Set(cmddata, WRITE_DATA, 10, 0, (char*)data);
	errcode = DIC_Command(hic, WRITE_FILE, cmddata);
	if(errcode != SCARD_S_SUCCESS){
		printf("Write Des key file error: %8x \n",errcode);
	}
       
    	//write 18 bytes 3DES key, note the struct
	// the offset is 10 when write the key
	memcpy(data,"\x02\x10\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F\x10",18);
	DIC_Set(cmddata, WRITE_DATA, 18, 10, (char*)data);
	errcode= DIC_Command(hic, WRITE_FILE, cmddata);
	if(errcode != SCARD_S_SUCCESS){
		printf("Write Des key file error: %8x \n",errcode);
	}

//================================des encrypt =8=========================================//
	printf("des: \n");
	printf(" Plain text: ");
	for(i= 0 ;i<dataLength;i++){
		printf("%02x ",DesData[i]);
	}
	printf("\n");
	DIC_Set(cmddata,DES_KEYFILEID,0,0x6F01,NULL);
	DIC_Set(cmddata,DES_KEYINDEX,0,0x01,NULL);
	DIC_Set(cmddata,DES_DATA,BY_ARRAY | dataLength,0,(char*)DesData);
	errcode = DIC_Command(hic, DESENC, cmddata);
	wRet = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, (char*)outData);
	printf("Cipher text: ");
	for(i = 0;i<wRet;i++){
		printf("%02x ",(BYTE)outData[i]);
	}
	printf("\n");	
		
 //====================================des decrypt =8===================================//  
   		
	DIC_Set(cmddata,DES_KEYFILEID,0,0x6F01,NULL);
	DIC_Set(cmddata,DES_KEYINDEX,0,0x01,NULL);
	DIC_Set(cmddata,DES_DATA,BY_ARRAY|wRet,0,outData);
	errcode = DIC_Command(hic, DESDEC, cmddata);
	wRet = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, (char*)DesData);
    printf(" Plain text: ");
	for(i= 0 ;i<wRet;i++){
		printf("%02x ",DesData[i]);
	}
	printf("\n");	

	//================================3des encrypt =8=========================================//
	printf("3des: \n");
	printf(" Plain text: ");
	for(i= 0 ;i<dataLength;i++){
		printf("%02x ",DesData[i]);
	}
	printf("\n");
	DIC_Set(cmddata,DES_KEYFILEID,0,0x6F01,NULL);
	DIC_Set(cmddata,DES_KEYINDEX,0,0x02,NULL); //3desd key id is 2
	DIC_Set(cmddata,DES_DATA,BY_ARRAY | dataLength,0,(char*)DesData);
	errcode = DIC_Command(hic, DESENC, cmddata);
	wRet = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, (char*)outData);
	printf("Cipher text: ");
	for(i = 0;i<wRet;i++){
		printf("%02x ",(BYTE)outData[i]);
	}
	printf("\n");	
		
 //====================================3des decrypt =8===================================//  
   		
	DIC_Set(cmddata,DES_KEYFILEID,0,0x6F01,NULL);
	DIC_Set(cmddata,DES_KEYINDEX,0,0x02,NULL);
	DIC_Set(cmddata,DES_DATA,BY_ARRAY|wRet,0,outData);
	errcode = DIC_Command(hic, DESDEC, cmddata);
	wRet = DIC_Get(cmddata, AFTER_ENCDEC_DATA, BY_ARRAY, (char*)DesData);
    printf(" Plain text: ");
	for(i= 0 ;i<wRet;i++){
		printf("%02x ",DesData[i]);
	}
	printf("\n");	
}
