/*DIC32L.cpp
* inline definations of DIC_Commands, for the easier usage
*/
#include "stdafx.h"
#include "Dic32L.h"
#include "dic32.h"
#include "iostream"


namespace hbx{
using namespace std;

inline int D32_RunExe(int hic,  //ROCKEY handle
					  WORD fileName, //exe file name
					  int dataSize,  //input data size
					  void *inData,  //inpute data
					  void *outData  //output data
					  )
{

	BYTE cmddata[1024];	
	int errcode;
	DIC_Set(cmddata, RUN_DATA, dataSize, fileName, (char*)inData);	// input data
	errcode = DIC_Command(hic, RUN, cmddata);
	DIC_Get(cmddata, RUN_DATA, BY_ARRAY, (char*)outData);	// output data
	return(errcode);					
}

/**format dongle
@param hic: ROCKEY handle
@return:  0 success or an error code
*/
inline int D32_FormatCard(int hic)
{
	int errcode;
	char cmddata[512];	// command buffer
	memset(cmddata,1,512);
	DIC_Set(cmddata, VOLUME, BY_ARRAY, 0, "DEFAULT VOL");
	DIC_Set(cmddata, ATR, BY_ARRAY, 0, "DEFAULT ATR");
	errcode = DIC_Command(hic, FORMAT_CARD, cmddata);
	if (errcode != SCARD_S_SUCCESS) return(errcode);	
	return(0);
}


//create file
int D32_CreateFile(int hic, USHORT id,byte attr,USHORT size, char *name, 
				   byte* g_progFile, byte _class)
{	
	if((attr&0x10) != 0){
		char cmddata1[8];
		for (int i=0;i<8;i++) cmddata1[i] = (char)0xff; // the default super password is 8 'FF'
		int errcode1 = DIC_Command(hic, CHECK_SUPER_PASS, cmddata1);
		ErrorShow(errcode1);
	}

	
	
	char cmddata[3500];	    // command buffer
	int errcode=1;
	
	//create file
	DIC_Set(cmddata, FILL, 256, 0, NULL);				// clear
	DIC_Set(cmddata, FILE_ID, BY_VALUE, id, NULL);		    	// file ID
	DIC_Set(cmddata, FILE_CLASS, BY_VALUE, _class, NULL);		// file class
	DIC_Set(cmddata, FILE_ATTRIBUTE, BY_VALUE, attr, NULL);	        // file attr
	DIC_Set(cmddata, FILE_PRIVILEGE, BY_VALUE, attr, NULL);		// security level
	DIC_Set(cmddata, FILE_SIZE, BY_VALUE,size, NULL);		// file size
	DIC_Set(cmddata, FILE_NAME, BY_ARRAY, 0, name);
	errcode = DIC_Command(hic, CREATE_FILE, cmddata);
	if (errcode!=0 || errcode==0xA0100001) {
		if((attr&0x10) != 0){
			DIC_Close(hic);
			DIC_Open(hic,NULL);
		}
		return errcode;
	}
	
	//write data
	if (g_progFile!=NULL) {
		DIC_Set(cmddata, WRITE_DATA, size, 0, (char*)g_progFile);
		errcode = DIC_Command(hic, WRITE_FILE, cmddata);
	}

	if((attr&0x10) != 0){
		DIC_Close(hic);
		DIC_Open(hic,NULL);
	}
	return errcode;
}

/**error code
@doc: error info
@param errcode: error code 
@author zhengyu 
*/
void ErrorShowExit(int errcode,char *place)
//show error and exit
{
	if (errcode==0) {
		return;
	}
	int errorSize;
	typedef struct{
		int error;
		char errorInfo[100];
	}oneError;

	oneError errorArray[]={
		SCARD_F_INTERNAL_ERROR,"Internal link check failure",
		SCARD_E_CANCELLED     ,"Operation is terminated by user",
		SCARD_E_INVALID_HANDLE,"Incorrect operation handle",
		SCARD_E_INVALID_PARAMETER,"Incorrect parameter",
		SCARD_E_INVALID_TARGET,   "Registered startup information lost or invalid ",
		SCARD_E_NO_MEMORY,        "Memory not enough for command completion ",
		SCARD_F_WAITED_TOO_LONG,"Internal overtime",
		SCARD_E_INSUFFICIENT_BUFFER,"User specified buffer is too small, not enough for the return data",
		SCARD_E_UNKNOWN_READER,"Unknown card reader",
		SCARD_E_TIMEOUT,"Over user specified time",
		SCARD_E_SHARING_VIOLATION,"Card occupied by other connection",
		SCARD_E_NO_SMARTCARD,"No card in the card reader",
		SCARD_E_UNKNOWN_CARD,"Unknown card type",
		SCARD_E_CANT_DISPOSE,"Card reader can not complete card eject operation",
		SCARD_E_PROTO_MISMATCH,"Current card does not support user specified communication protocol",
		SCARD_E_NOT_READY,"Card is not ready for receiving commands",
		SCARD_E_INVALID_VALUE,"Some variable values are not suitable",
		SCARD_E_SYSTEM_CANCELLED,"Operation is terminated by system, it may be necessary to log on again or shut down. ",
		SCARD_F_COMM_ERROR,"Internal communication error",
		SCARD_F_UNKNOWN_ERROR,"Unknown internal error",
		SCARD_E_INVALID_ATR,"Invalid vendor information",
		SCARD_E_NOT_TRANSACTED,"User attempts to terminate some process not available",
		SCARD_E_READER_UNAVAILABLE,"The specifiied card reader is not ready",
		SCARD_P_SHUTDOWN,"Operation is terminated, service program is allowed to exit",
		SCARD_E_PCI_TOO_SMALL,"PCI receiving buffer is too small",
		SCARD_E_READER_UNSUPPORTED,"Card reader drive does not support the current card reader",
		SCARD_E_DUPLICATE_READER,"card reader with the same name already exists",
		SCARD_E_CARD_UNSUPPORTED,"Card is not supported by the current card reader",
		SCARD_E_NO_SERVICE,"Smart card service is not enabled",
		SCARD_E_SERVICE_STOPPED,"Smart card service is disabled",
		SCARD_E_UNEXPECTED,"Some unexpected smart card error",
		SCARD_E_ICC_INSTALLATION,"Smart card software developer information not accessible",
		SCARD_E_ICC_CREATEORDER,"Smart card vendor information not accessible",
		SCARD_E_UNSUPPORTED_FEATURE,"Current smart card does not support the function user demanded",
		SCARD_E_DIR_NOT_FOUND,"The specified directory does not exist",
		SCARD_E_FILE_NOT_FOUND,"The specified file does not exist",
		SCARD_E_NO_DIR,"The specified directory is no longer a valid directory",
		SCARD_E_NO_FILE,"The specified file is no longer a valid file, no file selected",
		SCARD_E_NO_ACCESS,"The file denies any access",
		SCARD_E_WRITE_TOO_MANY,"Card is full, no more data write",
		SCARD_E_BAD_SEEK,"File pointer error",
		SCARD_E_INVALID_CHV,"PIN code error",
		SCARD_E_UNKNOWN_RES_MNG,"An unrecognized error code is returned from smart card service",
		SCARD_E_NO_SUCH_CERTIFICATE,"The certificate requested does not exist",
		SCARD_E_CERTIFICATE_UNAVAILABLE,"The certificate requested is not available",
		SCARD_E_NO_READERS_AVAILABLE,"No card reader found",
		SCARD_E_COMM_DATA_LOST,"Data loss during smart card communication, please try again",
		SCARD_E_NO_KEY_CONTAINER,"The key file requested does not exist",
		SCARD_W_UNSUPPORTED_CARD,"Card reader can not communicate with the card due to software developer information conflict",
		SCARD_W_UNRESPONSIVE_CARD,"Card does not respond to reset",
		SCARD_W_UNPOWERED_CARD,"Card no power",
		SCARD_W_RESET_CARD,"Card is reset, shared information is invalid",
		SCARD_W_REMOVED_CARD,"Card is removed",
		SCARD_W_SECURITY_VIOLATION,"Access is denied due to security rules",
		SCARD_W_WRONG_CHV,"PIN code is not verified, access is denied",
		SCARD_W_CHV_BLOCKED,"Max. PIN code retry number is reached, access is denied",
		SCARD_W_EOF,"The last smart card file is reached, no more file available",
		SCARD_W_CANCELLED_BY_USER,"Operation is terminated by user",
		SCARD_W_CARD_NOT_AUTHENTICATED,"Smart card PIN is not set",
		SCARD_E_FILE_EXISTS,"File already exists",
		SCARD_E_EPROM_ERROR,"Card-resident memory operation error",
		SCARD_E_INVALID_CLA,"User specified CLA is invalid",
		SCARD_E_INVALID_INS,"User specified INS is invalid",
		SCARD_E_VM_ADDRESS_ERROR,"Virtual machine address out ranged/abnormal ",
		SCARD_E_ZERO_DIVIDE,"0 division error",
		SCARD_E_WRONG_POSITION,"Card is not inserted to the proper place",
		SCARD_E_UNKNOWN_STATE,"Card is in an unknown status",
		SCARD_E_CARD_NOT_OPENED,"Card is not opened",
		SCARD_E_UNKNOWN_COMMAND,"Unknown command",
		SCARD_E_ZERO_TRYTIME,"Super password reset number is 0",
		SCARD_E_TOO_MANY_DEVICE,"Too many devices are opened",
		SCARD_E_INVALID_INSTRUCTION,"Invalid instruction.",
		SCARD_W_RESPONSE,"card still have more data to return ",
		SCARD_E_FILE_CREATE_FAILED,"Virtual card file creation failure",
		SCARD_E_FILE_OPEN_FAILED,"Virtual card file open failure",		
	};
	errorSize = sizeof(errorArray)/sizeof(oneError);
	for(int i = 0;i<errorSize;i++){
		if(errorArray[i].error == errcode){
			printf("Error:%8x,%s.%s\n",errorArray[i].error,errorArray[i].errorInfo,place);
			int a;cin>>a;
			exit(0);
		}
	}	
} 


/**show error info
@doc: input a error code
@param errcode: output the error info
		place:  where the error occure
@author zhengyu 
*/
int ErrorShow(int errcode,char *place)
{
	if (errcode==0) {
		return errcode;
	}

	int errorSize;
	typedef struct{
		int error;
		char errorInfo[100];
	}oneError;

	oneError errorArray[]={
		SCARD_F_INTERNAL_ERROR,"Internal link check failure",
		SCARD_E_CANCELLED     ,"Operation is terminated by user",
		SCARD_E_INVALID_HANDLE,"Incorrect operation handle",
		SCARD_E_INVALID_PARAMETER,"Incorrect parameter",
		SCARD_E_INVALID_TARGET,   "Registered startup information lost or invalid ",
		SCARD_E_NO_MEMORY,        "Memory not enough for command completion ",
		SCARD_F_WAITED_TOO_LONG,"Internal overtime",
		SCARD_E_INSUFFICIENT_BUFFER,"User specified buffer is too small, not enough for the return data",
		SCARD_E_UNKNOWN_READER,"Unknown card reader",
		SCARD_E_TIMEOUT,"Over user specified time",
		SCARD_E_SHARING_VIOLATION,"Card occupied by other connection",
		SCARD_E_NO_SMARTCARD,"No card in the card reader",
		SCARD_E_UNKNOWN_CARD,"Unknown card type",
		SCARD_E_CANT_DISPOSE,"Card reader can not complete card eject operation",
		SCARD_E_PROTO_MISMATCH,"Current card does not support user specified communication protocol",
		SCARD_E_NOT_READY,"Card is not ready for receiving commands",
		SCARD_E_INVALID_VALUE,"Some variable values are not suitable",
		SCARD_E_SYSTEM_CANCELLED,"Operation is terminated by system, it may be necessary to log on again or shut down. ",
		SCARD_F_COMM_ERROR,"Internal communication error",
		SCARD_F_UNKNOWN_ERROR,"Unknown internal error",
		SCARD_E_INVALID_ATR,"Invalid vendor information",
		SCARD_E_NOT_TRANSACTED,"User attempts to terminate some process not available",
		SCARD_E_READER_UNAVAILABLE,"The specifiied card reader is not ready",
		SCARD_P_SHUTDOWN,"Operation is terminated, service program is allowed to exit",
		SCARD_E_PCI_TOO_SMALL,"PCI receiving buffer is too small",
		SCARD_E_READER_UNSUPPORTED,"Card reader drive does not support the current card reader",
		SCARD_E_DUPLICATE_READER,"card reader with the same name already exists",
		SCARD_E_CARD_UNSUPPORTED,"Card is not supported by the current card reader",
		SCARD_E_NO_SERVICE,"Smart card service is not enabled",
		SCARD_E_SERVICE_STOPPED,"Smart card service is disabled",
		SCARD_E_UNEXPECTED,"Some unexpected smart card error",
		SCARD_E_ICC_INSTALLATION,"Smart card software developer information not accessible",
		SCARD_E_ICC_CREATEORDER,"Smart card vendor information not accessible",
		SCARD_E_UNSUPPORTED_FEATURE,"Current smart card does not support the function user demanded",
		SCARD_E_DIR_NOT_FOUND,"The specified directory does not exist",
		SCARD_E_FILE_NOT_FOUND,"The specified file does not exist",
		SCARD_E_NO_DIR,"The specified directory is no longer a valid directory",
		SCARD_E_NO_FILE,"The specified file is no longer a valid file, no file selected",
		SCARD_E_NO_ACCESS,"The file denies any access",
		SCARD_E_WRITE_TOO_MANY,"Card is full, no more data write",
		SCARD_E_BAD_SEEK,"File pointer error",
		SCARD_E_INVALID_CHV,"PIN code error",
		SCARD_E_UNKNOWN_RES_MNG,"An unrecognized error code is returned from smart card service",
		SCARD_E_NO_SUCH_CERTIFICATE,"The certificate requested does not exist",
		SCARD_E_CERTIFICATE_UNAVAILABLE,"The certificate requested is not available",
		SCARD_E_NO_READERS_AVAILABLE,"No card reader found",
		SCARD_E_COMM_DATA_LOST,"Data loss during smart card communication, please try again",
		SCARD_E_NO_KEY_CONTAINER,"The key file requested does not exist",
		SCARD_W_UNSUPPORTED_CARD,"Card reader can not communicate with the card due to software developer information conflict",
		SCARD_W_UNRESPONSIVE_CARD,"Card does not respond to reset",
		SCARD_W_UNPOWERED_CARD,"Card no power",
		SCARD_W_RESET_CARD,"Card is reset, shared information is invalid",
		SCARD_W_REMOVED_CARD,"Card is removed",
		SCARD_W_SECURITY_VIOLATION,"Access is denied due to security rules",
		SCARD_W_WRONG_CHV,"PIN code is not verified, access is denied",
		SCARD_W_CHV_BLOCKED,"Max. PIN code retry number is reached, access is denied",
		SCARD_W_EOF,"The last smart card file is reached, no more file available",
		SCARD_W_CANCELLED_BY_USER,"Operation is terminated by user",
		SCARD_W_CARD_NOT_AUTHENTICATED,"Smart card PIN is not set",
		SCARD_E_FILE_EXISTS,"File already exists",
		SCARD_E_EPROM_ERROR,"Card-resident memory operation error",
		SCARD_E_INVALID_CLA,"User specified CLA is invalid",
		SCARD_E_INVALID_INS,"User specified INS is invalid",
		SCARD_E_VM_ADDRESS_ERROR,"Virtual machine address out ranged/abnormal ",
		SCARD_E_ZERO_DIVIDE,"0 division error",
		SCARD_E_WRONG_POSITION,"Card is not inserted to the proper place",
		SCARD_E_UNKNOWN_STATE,"Card is in an unknown status",
		SCARD_E_CARD_NOT_OPENED,"Card is not opened",
		SCARD_E_UNKNOWN_COMMAND,"Unknown command",
		SCARD_E_ZERO_TRYTIME,"Super password reset number is 0",
		SCARD_E_TOO_MANY_DEVICE,"Too many devices are opened",
		SCARD_E_INVALID_INSTRUCTION,"Invalid instruction.",
		SCARD_W_RESPONSE,"card still have more data to return ",
		SCARD_E_FILE_CREATE_FAILED,"Virtual card file creation failure",
		SCARD_E_FILE_OPEN_FAILED,"Virtual card file open failure",	
	};
	errorSize = sizeof(errorArray)/sizeof(oneError);
	for(int i = 0;i<errorSize;i++){
		if(errorArray[i].error == errcode){
			printf("Error:%8x,%s.%s\n\n",errorArray[i].error,errorArray[i].errorInfo,place);
			break;
		
		}
	}
	
	return errcode;
} 

}//end namespace HBX